@@ -27,11 +27,7 @@
 
 #include	<iostream>
 
-#ifndef AQSIS_SYSTEM_MACOSX
 #include	<sstream>
-#else
-#include	<strstream>
-#endif
 #include	<ctype.h>
 
 #include	"shadervm.h"
@@ -725,12 +721,7 @@ void CqShaderVM::DefaultSurface()
                                pop Ci \
                                ";
 
-#ifndef AQSIS_SYSTEM_MACOSX
     std::stringstream defStream(pDefSurfaceShader);
-#else
-    std::strstream defStream;
-    defStream  << pDefSurfaceShader << std::ends;
-#endif
 
     LoadProgram(&defStream);
 }
@@ -1312,7 +1303,7 @@ void CqShaderVM::ExecuteInit()
     boost::shared_ptr<IqShaderExecEnv> pOldEnv = m_pEnv;
 
     boost::shared_ptr<IqShaderExecEnv> Env(new CqShaderExecEnv);
-    Env->Initialise( 1, 1, 0, 0, this, m_Uses );
+    Env->Initialise( 1, 1, 0, boost::shared_ptr<IqTransform>(), this, m_Uses );
     Initialise( 1, 1, Env );
 
     // Execute the init program.
