@@ -34,12 +34,6 @@
 
 START_NAMESPACE( Aqsis )
 
-#ifdef AQSIS_SYSTEM_MACOSX 
-// Workaround for Mac OS X gcc 2.95 compiler error -
-// "Fixup [linenumber] too large for field width of 16 bits"
-#pragma CC_OPT_OFF
-#endif
-
 void CqShaderVM::SO_nop()
 {}
 
@@ -101,7 +95,8 @@ void CqShaderVM::SO_ipushv()
         std::cerr << critical << "Attempt to index a non array variable" << std::endl;
         return ;
     }
-    RESULT(pVar->Type(), pVar->Class());
+	//If either the value or the index is varying, so must the result be.
+	RESULT(pVar->Type(), (pVar->Size()>1 || A->Size()>1)?class_varying:class_uniform);
     TqInt ext = m_pEnv->GridSize();
     TqInt i;
     for ( i = 0; i < ext; i++ )
@@ -1177,9 +1172,5 @@ void CqShaderVM::SO_pow()
     FUNC2( type_float, m_pEnv->SO_pow );
 }
 
-#ifdef AQSIS_SYSTEM_MACOSX
-#pragma CC_OPT_RESTORE
-#endif
-
 END_NAMESPACE( Aqsis )
 //---------------------------------------------------------------------
