@@ -285,15 +285,22 @@ TqInt	CqParseNodeUnresolvedCall::TypeCheck( TqInt* pTypes, TqInt Count, TqBool C
     TqInt NewType = Type_Nil;
 
     // TypeCheck the args, we don't know what types will be needed
-    // later, so there is little else we can do.
+    // later, so there is little else we can do. While doing so, rebuild the params string for the
+	// external declaration in the .slx
+	CqString strArgTypes("");
     CqParseNode *pArg = m_pChild;
     while (pArg != NULL)
     {
         CqParseNode *pNext = pArg->pNext();
         pArg->TypeCheck( pAllTypes(), Type_Last - 1 );
+		strArgTypes+=CqParseNode::TypeIdentifier(pArg->ResType());
         pArg = pNext;
     };
 
+	// Store the newly decided parameter list string. This now reflects the actual parameter types 
+	// passed in.
+	m_aFuncDef.SetstrParams(strArgTypes);
+
     // If we have no type set yet we take the first type given as an option
     if(m_aFuncDef.Type() == Type_Nil || !CheckOnly)
     {
@@ -515,11 +522,11 @@ TqInt	CqParseNodeOp::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
 
 TqInt	CqParseNodeRelOp::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
 {
-    // Type check the operands first.
-    CqParseNode * pExpr1 = m_pChild;
-    CqParseNode* pExpr2 = pExpr1->pNext();
-    pExpr1->TypeCheck( pAllTypes(), Type_Last - 1, CheckOnly );
-    pExpr2->TypeCheck( pAllTypes(), Type_Last - 1, CheckOnly );
+    TqInt RelType;
+    RelType = CqParseNodeOp::TypeCheck( pAllTypes(), Type_Last - 1, CheckOnly );  
+
+    if( RelType == Type_Nil)
+    	return( RelType );
 
     // See if float is a requested type.
     TqInt NewType;
@@ -540,7 +547,7 @@ TqInt	CqParseNodeRelOp::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly
         strErr += " : ";
         strErr += LineNo();
         strErr += " : ";
-        strErr += "Relative operators only return float.";
+        strErr += "Relational can operators only return float.";
         throw( strErr );
         return ( Type_Nil );
     }
