@@ -125,13 +125,13 @@ public:
     /** Add a lightsource to the current available list.
      * \param pL a pointer to the new lightsource.
      */
-    void	AddLightsource( CqLightsource* pL )
+    void	AddLightsource( const boost::shared_ptr<CqLightsource>& pL )
     {
         // Check if the ligthsource is already active
-        std::vector<CqLightsource*>::iterator end = m_apLightsources.end();
-        for ( std::vector<CqLightsource*>::iterator i = m_apLightsources.begin(); i != end; i++ )
+        std::vector<boost::shared_ptr<CqLightsource> >::iterator end = m_apLightsources.end();
+        for ( std::vector<boost::shared_ptr<CqLightsource> >::iterator i = m_apLightsources.begin(); i != end; i++ )
         {
-            if ( ( *i ) == pL )
+            if ( ( i->get() ) == pL.get() )
                 return ;
         }
         m_apLightsources.push_back( pL );
@@ -139,13 +139,13 @@ public:
     /** Remove a lightsource from the current available list.
      * \param pL a pointer to the lightsource to remove.
      */
-    void	RemoveLightsource( CqLightsource* pL )
+    void	RemoveLightsource( const boost::shared_ptr<CqLightsource>& pL )
     {
         // Check if the ligthsource is in the active list.
-        std::vector<CqLightsource*>::iterator end = m_apLightsources.end();
-        for ( std::vector<CqLightsource*>::iterator i = m_apLightsources.begin(); i != end; i++ )
+        std::vector<boost::shared_ptr<CqLightsource> >::iterator end = m_apLightsources.end();
+        for ( std::vector<boost::shared_ptr<CqLightsource> >::iterator i = m_apLightsources.begin(); i != end; i++ )
         {
-            if ( *i == pL )
+            if ( i->get() == pL.get() )
             {
                 m_apLightsources.erase( i );
                 return ;
@@ -155,7 +155,7 @@ public:
     /** Get a reference to the lightsource list.
      * \return a reference to the vector of lightsource pointers.
      */
-    virtual const	std::vector<CqLightsource*>&	apLights() const
+    virtual const	std::vector<boost::shared_ptr<CqLightsource> >&	apLights() const
     {
         return ( m_apLightsources );
     }
@@ -169,51 +169,51 @@ public:
         GetIntegerAttributeWrite( "System", "Orientation" ) [ 0 ] = ( co ) ? 1 : 0;
     }
 
-    virtual IqShader*	pshadDisplacement( TqFloat time ) const
+    virtual boost::shared_ptr<IqShader> pshadDisplacement( TqFloat time ) const
     {
         return ( m_pshadDisplacement );
     }
-    virtual void	SetpshadDisplacement( IqShader* pshadDisplacement, TqFloat time )
+    virtual void SetpshadDisplacement( const boost::shared_ptr<IqShader>& pshadDisplacement, TqFloat time )
     {
         m_pshadDisplacement = pshadDisplacement;
     }
-    virtual IqShader*	pshadAreaLightSource( TqFloat time ) const
+    virtual boost::shared_ptr<IqShader> pshadAreaLightSource( TqFloat time ) const
     {
         return ( m_pshadAreaLightSource );
     }
-    virtual void	SetpshadAreaLightSource( IqShader* pshadAreaLightSource, TqFloat time )
+    virtual void SetpshadAreaLightSource( const boost::shared_ptr<IqShader>& pshadAreaLightSource, TqFloat time )
     {
         m_pshadAreaLightSource = pshadAreaLightSource;
     }
-    virtual IqShader*	pshadSurface( TqFloat time ) const
+    virtual boost::shared_ptr<IqShader> pshadSurface( TqFloat time ) const
     {
         return ( m_pshadSurface );
     }
-    virtual void	SetpshadSurface( IqShader* pshadSurface, TqFloat time )
+    virtual void SetpshadSurface( const boost::shared_ptr<IqShader>& pshadSurface, TqFloat time )
     {
         m_pshadSurface = pshadSurface;
     }
-    virtual IqShader*	pshadAtmosphere( TqFloat time ) const
+    virtual boost::shared_ptr<IqShader> pshadAtmosphere( TqFloat time ) const
     {
         return ( m_pshadAtmosphere );
     }
-    virtual void	SetpshadAtmosphere( IqShader* pshadAtmosphere, TqFloat time )
+    virtual void SetpshadAtmosphere( const boost::shared_ptr<IqShader>& pshadAtmosphere, TqFloat time )
     {
         m_pshadAtmosphere = pshadAtmosphere;
     }
-    virtual IqShader*	pshadExteriorVolume( TqFloat time ) const
+    virtual boost::shared_ptr<IqShader> pshadExteriorVolume( TqFloat time ) const
     {
         return ( m_pshadExteriorVolume );
     }
-    virtual void	SetpshadExteriorVolume( IqShader* pshadExteriorVolume, TqFloat time )
+    virtual void SetpshadExteriorVolume( const boost::shared_ptr<IqShader>& pshadExteriorVolume, TqFloat time )
     {
         m_pshadExteriorVolume = pshadExteriorVolume;
     }
-    virtual IqShader*	pshadAreaInteriorVolume( TqFloat time ) const
+    virtual boost::shared_ptr<IqShader> pshadAreaInteriorVolume( TqFloat time ) const
     {
         return ( m_pshadInteriorVolume );
     }
-    virtual void	SetpshadInteriorVolume( IqShader* pshadInteriorVolume, TqFloat time )
+    virtual void SetpshadInteriorVolume( const boost::shared_ptr<IqShader>& pshadInteriorVolume, TqFloat time )
     {
         m_pshadInteriorVolume = pshadInteriorVolume;
     }
@@ -488,15 +488,15 @@ private:
 
     CqHashTable	m_aAttributes;						///< a vector of user defined attribute pointers.
 
-    IqShader*	m_pshadDisplacement;				///< a pointer to the current displacement shader.
-    IqShader*	m_pshadAreaLightSource;				///< a pointer to the current area ligthsource shader.
-    IqShader*	m_pshadSurface;						///< a pointer to the current surface shader.
-    IqShader*	m_pshadAtmosphere;					///< a pointer to the current atmosphere shader.
-    IqShader*	m_pshadInteriorVolume;				///< a pointer to the current interior shader.
-    IqShader*	m_pshadExteriorVolume;				///< a pointer to the current exterior shader.
+    boost::shared_ptr<IqShader> m_pshadDisplacement;            ///< a pointer to the current displacement shader.
+    boost::shared_ptr<IqShader> m_pshadAreaLightSource;         ///< a pointer to the current area ligthsource shader.
+    boost::shared_ptr<IqShader> m_pshadSurface;                 ///< a pointer to the current surface shader.
+    boost::shared_ptr<IqShader> m_pshadAtmosphere;              ///< a pointer to the current atmosphere shader.
+    boost::shared_ptr<IqShader> m_pshadInteriorVolume;          ///< a pointer to the current interior shader.
+    boost::shared_ptr<IqShader> m_pshadExteriorVolume;          ///< a pointer to the current exterior shader.
 
     CqTrimLoopArray m_TrimLoops;					///< the array of closed trimcurve loops.
-    std::vector<CqLightsource*> m_apLightsources;	///< a vector of currently available lightsources.
+    std::vector<boost::shared_ptr<CqLightsource> > m_apLightsources;	///< a vector of currently available lightsources.
 
     std::list<CqAttributes*>::iterator	m_StackIterator;	///< the index of this attribute state in the global stack, used for destroying when last reference is removed.
 }
