@@ -33,6 +33,8 @@
 #include	"shaders.h"
 #include	"trimcurve.h"
 
+#include	"mpdump.h"
+
 START_NAMESPACE( Aqsis )
 
 
@@ -112,15 +114,15 @@ void CqMicroPolyGrid::Initialise( TqInt cu, TqInt cv, const boost::shared_ptr<Cq
 
     /// \note This should delete through the interface that created it.
 
-    m_pShaderExecEnv->Initialise( cu, cv, pSurface->pAttributes(), pSurface->pTransform(), pSurface->pAttributes() ->pshadSurface(QGetRenderContext()->Time()), lUses );
+    m_pShaderExecEnv->Initialise( cu, cv, pSurface->pAttributes(), pSurface->pTransform(), pSurface->pAttributes()->pshadSurface(QGetRenderContext()->Time()).get(), lUses );
 
-    IqShader* pshadSurface = pSurface ->pAttributes() ->pshadSurface(QGetRenderContext()->Time());
-    IqShader* pshadDisplacement = pSurface ->pAttributes() ->pshadDisplacement(QGetRenderContext()->Time());
-    IqShader* pshadAtmosphere = pSurface ->pAttributes() ->pshadAtmosphere(QGetRenderContext()->Time());
+    boost::shared_ptr<IqShader> pshadSurface = pSurface ->pAttributes() ->pshadSurface(QGetRenderContext()->Time());
+    boost::shared_ptr<IqShader> pshadDisplacement = pSurface ->pAttributes() ->pshadDisplacement(QGetRenderContext()->Time());
+    boost::shared_ptr<IqShader> pshadAtmosphere = pSurface ->pAttributes() ->pshadAtmosphere(QGetRenderContext()->Time());
 
-    if ( NULL != pshadSurface ) pshadSurface->Initialise( cu, cv, m_pShaderExecEnv );
-    if ( NULL != pshadDisplacement ) pshadDisplacement->Initialise( cu, cv, m_pShaderExecEnv );
-    if ( NULL != pshadAtmosphere ) pshadAtmosphere->Initialise( cu, cv, m_pShaderExecEnv );
+    if ( pshadSurface ) pshadSurface->Initialise( cu, cv, m_pShaderExecEnv );
+    if ( pshadDisplacement ) pshadDisplacement->Initialise( cu, cv, m_pShaderExecEnv );
+    if ( pshadAtmosphere ) pshadAtmosphere->Initialise( cu, cv, m_pShaderExecEnv );
 
     // Initialise the local/public culled variable.
     m_CulledPolys.SetSize( ( cu + 1 ) * ( cv + 1 ) );
@@ -256,9 +258,9 @@ void CqMicroPolyGrid::Shade()
 
     CqStats& theStats = QGetRenderContext() ->Stats();
 
-    IqShader* pshadSurface = pSurface() ->pAttributes() ->pshadSurface(QGetRenderContext()->Time());
-    IqShader* pshadDisplacement = pSurface() ->pAttributes() ->pshadDisplacement(QGetRenderContext()->Time());
-    IqShader* pshadAtmosphere = pSurface() ->pAttributes() ->pshadAtmosphere(QGetRenderContext()->Time());
+    boost::shared_ptr<IqShader> pshadSurface = pSurface() ->pAttributes() ->pshadSurface(QGetRenderContext()->Time());
+    boost::shared_ptr<IqShader> pshadDisplacement = pSurface() ->pAttributes() ->pshadDisplacement(QGetRenderContext()->Time());
+    boost::shared_ptr<IqShader> pshadAtmosphere = pSurface() ->pAttributes() ->pshadAtmosphere(QGetRenderContext()->Time());
 
     TqInt lUses = pSurface() ->Uses();
     TqInt gs = GridSize();
@@ -407,7 +409,7 @@ void CqMicroPolyGrid::Shade()
         }
     }
 
-    if ( pshadDisplacement != 0 )
+    if ( pshadDisplacement )
     {
         theStats.DisplacementTimer().Start();
         pshadDisplacement->Evaluate( m_pShaderExecEnv );
@@ -442,8 +444,10 @@ void CqMicroPolyGrid::Shade()
 
     // Now shade the grid.
     theStats.SurfaceTimer().Start();
-    if ( NULL != pshadSurface )
+    if ( pshadSurface )
     {
+		//boost::shared_ptr<CqSurface> surf(pSurface());
+		m_pShaderExecEnv->SetCurrentSurface(pSurface());
         pshadSurface->Evaluate( m_pShaderExecEnv );
     }
     theStats.SurfaceTimer().Stop();
@@ -475,7 +479,7 @@ void CqMicroPolyGrid::Shade()
         }
     }
     // Now perform atmosphere shading
-    if ( pshadAtmosphere != 0 )
+    if ( pshadAtmosphere )
     {
         theStats.AtmosphereTimer().Start();
         pshadAtmosphere->Evaluate( m_pShaderExecEnv );
@@ -495,7 +499,7 @@ void CqMicroPolyGrid::Shade()
 
 void CqMicroPolyGrid::TransferOutputVariables()
 {
-    IqShader* pShader = this->pAttributes()->pshadSurface(QGetRenderContext()->Time());
+    boost::shared_ptr<IqShader> pShader = this->pAttributes()->pshadSurface(QGetRenderContext()->Time());
 
     // Only bother transferring ones that have been used in a RiDisplay request.
     std::map<std::string, CqRenderer::SqOutputDataEntry>& outputVars = QGetRenderContext()->GetMapOfOutputDataEntries();
@@ -709,19 +713,26 @@ void CqMicroPolyGrid::Split( CqImageBuffer* pImage, long xmin, long xmax, long y
             TqBool fTrimmed = TqFalse;
             if ( bCanBeTrimmed )
             {
-                TqFloat fu, fv;
-                pVar(EnvVars_u) ->GetFloat( fu, iIndex );
-                pVar(EnvVars_v) ->GetFloat( fv, iIndex );
-                TqBool fTrimA = pSurface() ->bIsPointTrimmed( CqVector2D( fu, fv ) );
-                pVar(EnvVars_u) ->GetFloat( fu, iIndex + 1 );
-                pVar(EnvVars_v) ->GetFloat( fv, iIndex + 1 );
-                TqBool fTrimB = pSurface() ->bIsPointTrimmed( CqVector2D( fu, fv ) );
-                pVar(EnvVars_u) ->GetFloat( fu, iIndex + cu + 2 );
-                pVar(EnvVars_v) ->GetFloat( fv, iIndex + cu + 2 );
-                TqBool fTrimC = pSurface() ->bIsPointTrimmed( CqVector2D( fu, fv ) );
-                pVar(EnvVars_u) ->GetFloat( fu, iIndex + cu + 1 );
-                pVar(EnvVars_v) ->GetFloat( fv, iIndex + cu + 1 );
-                TqBool fTrimD = pSurface() ->bIsPointTrimmed( CqVector2D( fu, fv ) );
+                TqFloat u1, v1, u2, v2, u3, v3, u4, v4;
+
+                pVar(EnvVars_u) ->GetFloat( u1, iIndex );
+                pVar(EnvVars_v) ->GetFloat( v1, iIndex );
+                pVar(EnvVars_u) ->GetFloat( u2, iIndex + 1 );
+                pVar(EnvVars_v) ->GetFloat( v2, iIndex + 1 );
+                pVar(EnvVars_u) ->GetFloat( u3, iIndex + cu + 2 );
+                pVar(EnvVars_v) ->GetFloat( v3, iIndex + cu + 2 );
+                pVar(EnvVars_u) ->GetFloat( u4, iIndex + cu + 1 );
+                pVar(EnvVars_v) ->GetFloat( v4, iIndex + cu + 1 );
+                
+				CqVector2D vecA(u1, v1);
+				CqVector2D vecB(u2, v2);
+				CqVector2D vecC(u3, v3);
+				CqVector2D vecD(u4, v4);
+
+				TqBool fTrimA = pSurface() ->bIsPointTrimmed( vecA );
+                TqBool fTrimB = pSurface() ->bIsPointTrimmed( vecB );
+                TqBool fTrimC = pSurface() ->bIsPointTrimmed( vecC );
+                TqBool fTrimD = pSurface() ->bIsPointTrimmed( vecD );
 
                 if ( bOutside )
                 {
@@ -731,9 +742,19 @@ void CqMicroPolyGrid::Split( CqImageBuffer* pImage, long xmin, long xmax, long y
                     fTrimD = !fTrimD;
                 }
 
-                // If al points are trimmed discard the MPG
+                // If all points are trimmed, need to check if the MP spans the trim curve at all, if not, then
+				// we can discard it altogether.
                 if ( fTrimA && fTrimB && fTrimC && fTrimD )
-                    continue;
+				{
+					if(!pSurface()->bIsLineIntersecting(vecA, vecB) && 
+					   !pSurface()->bIsLineIntersecting(vecB, vecC) && 
+					   !pSurface()->bIsLineIntersecting(vecC, vecD) && 
+					   !pSurface()->bIsLineIntersecting(vecD, vecA) )
+					{
+						STATS_INC( MPG_trimmedout );
+						continue;
+					}
+				}
 
                 // If any points are trimmed mark the MPG as needing to be trim checked.
                 //fTrimmed = fTrimA || fTrimB || fTrimC || fTrimD;
@@ -758,7 +779,7 @@ void CqMicroPolyGrid::Split( CqImageBuffer* pImage, long xmin, long xmax, long y
                 pNew->SetIndex( iIndex );
                 if ( fTrimmed ) pNew->MarkTrimmed();
                 pNew->Initialise();
-                pNew->GetTotalBound( TqTrue );
+                pNew->CalculateTotalBound();
                 pImage->AddMPG( pNew );
             }
 
@@ -924,19 +945,26 @@ void CqMotionMicroPolyGrid::Split( CqImageBuffer* pImage, long xmin, long xmax,
             TqBool fTrimmed = TqFalse;
             if ( bCanBeTrimmed )
             {
-                TqFloat fu, fv;
-                pGridA->pVar(EnvVars_u) ->GetFloat( fu, iIndex );
-                pGridA->pVar(EnvVars_v) ->GetFloat( fv, iIndex );
-                TqBool fTrimA = pSurface() ->bIsPointTrimmed( CqVector2D( fu, fv ) );
-                pGridA->pVar(EnvVars_u) ->GetFloat( fu, iIndex + 1 );
-                pGridA->pVar(EnvVars_v) ->GetFloat( fv, iIndex + 1 );
-                TqBool fTrimB = pSurface() ->bIsPointTrimmed( CqVector2D( fu, fv ) );
-                pGridA->pVar(EnvVars_u) ->GetFloat( fu, iIndex + cu + 2 );
-                pGridA->pVar(EnvVars_v) ->GetFloat( fv, iIndex + cu + 2 );
-                TqBool fTrimC = pSurface() ->bIsPointTrimmed( CqVector2D( fu, fv ) );
-                pGridA->pVar(EnvVars_u) ->GetFloat( fu, iIndex + cu + 1 );
-                pGridA->pVar(EnvVars_v) ->GetFloat( fv, iIndex + cu + 1 );
-                TqBool fTrimD = pSurface() ->bIsPointTrimmed( CqVector2D( fu, fv ) );
+                TqFloat u1, v1, u2, v2, u3, v3, u4, v4;
+
+                pGridA->pVar(EnvVars_u) ->GetFloat( u1, iIndex );
+                pGridA->pVar(EnvVars_v) ->GetFloat( v1, iIndex );
+                pGridA->pVar(EnvVars_u) ->GetFloat( u2, iIndex + 1 );
+                pGridA->pVar(EnvVars_v) ->GetFloat( v2, iIndex + 1 );
+                pGridA->pVar(EnvVars_u) ->GetFloat( u3, iIndex + cu + 2 );
+                pGridA->pVar(EnvVars_v) ->GetFloat( v3, iIndex + cu + 2 );
+                pGridA->pVar(EnvVars_u) ->GetFloat( u4, iIndex + cu + 1 );
+                pGridA->pVar(EnvVars_v) ->GetFloat( v4, iIndex + cu + 1 );
+                
+				CqVector2D vecA(u1, v1);
+				CqVector2D vecB(u2, v2);
+				CqVector2D vecC(u3, v3);
+				CqVector2D vecD(u4, v4);
+
+				TqBool fTrimA = pSurface() ->bIsPointTrimmed( vecA );
+                TqBool fTrimB = pSurface() ->bIsPointTrimmed( vecB );
+                TqBool fTrimC = pSurface() ->bIsPointTrimmed( vecC );
+                TqBool fTrimD = pSurface() ->bIsPointTrimmed( vecD );
 
                 if ( bOutside )
                 {
@@ -946,9 +974,19 @@ void CqMotionMicroPolyGrid::Split( CqImageBuffer* pImage, long xmin, long xmax,
                     fTrimD = !fTrimD;
                 }
 
-                // If al points are trimmed discard the MPG
+                // If all points are trimmed, need to check if the MP spans the trim curve at all, if not, then
+				// we can discard it altogether.
                 if ( fTrimA && fTrimB && fTrimC && fTrimD )
-                    continue;
+				{
+					if(!pSurface()->bIsLineIntersecting(vecA, vecB) && 
+					   !pSurface()->bIsLineIntersecting(vecB, vecC) && 
+					   !pSurface()->bIsLineIntersecting(vecC, vecD) && 
+					   !pSurface()->bIsLineIntersecting(vecD, vecA) )
+					{
+						STATS_INC( MPG_trimmedout );
+						continue;
+					}
+				}
 
                 // If any points are trimmed mark the MPG as needing to be trim checked.
                 //fTrimmed = fTrimA || fTrimB || fTrimC || fTrimD;
@@ -983,7 +1021,7 @@ void CqMotionMicroPolyGrid::Split( CqImageBuffer* pImage, long xmin, long xmax,
 /** Default constructor
  */
 
-CqMicroPolygon::CqMicroPolygon() : m_pGrid( 0 ), m_Flags( 0 )
+CqMicroPolygon::CqMicroPolygon() : m_pGrid( 0 ), m_Flags( 0 ), m_pHitTestCache( 0 )
 {
     STATS_INC( MPG_allocated );
     STATS_INC( MPG_current );
@@ -1100,7 +1138,7 @@ void CqMicroPolygon::Initialise()
 
 TqBool CqMicroPolygon::fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const
 {
-    // Check against each line of the quad, if outside any then point is outside MPG, therefore early exit.
+/*    // Check against each line of the quad, if outside any then point is outside MPG, therefore early exit.
     const CqVector3D& pA = PointA();
     const CqVector3D& pB = PointB();
     TqFloat x = vecP.x(), y = vecP.y();
@@ -1132,17 +1170,152 @@ TqBool CqMicroPolygon::fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloa
     Depth = ( D - ( vecN.x() * vecP.x() ) - ( vecN.y() * vecP.y() ) ) / vecN.z();
 
     return ( TqTrue );
+*/
+
+	// AGG - optimised version of above.
+	TqFloat x = vecP.x(), y = vecP.y();
+
+	// start with the edge that failed last time to get the most benefit
+	// from an early exit.
+	int e = m_pHitTestCache->m_LastFailedEdge;
+	int prev = e - 1;
+	if(prev < 0) prev = 3;
+	for(int i=0; i<4; ++i)
+	{
+		// test which side of the edge the sample point lies.
+		// the first two edges are tested with <= and the second two with <
+		// this is so every sample point lies on exactly one side of the edge,
+		// ie if it is exactly coincident with the edge it can't be on both
+		// or neither sides.
+		if(e & 2)
+//		if( m_pHitTestCache->m_Y[e] > m_pHitTestCache->m_Y[prev] ||
+//			( m_pHitTestCache->m_Y[e] == m_pHitTestCache->m_Y[prev] &&
+//			  m_pHitTestCache->m_X[e] > m_pHitTestCache->m_X[prev] ) )
+		{
+			if( (( y - m_pHitTestCache->m_Y[e]) * m_pHitTestCache->m_YMultiplier[e] ) -
+					(( x - m_pHitTestCache->m_X[e]) * m_pHitTestCache->m_XMultiplier[e] ) < 0)
+			{
+				m_pHitTestCache->m_LastFailedEdge = e;
+				return TqFalse;
+			}
+		}
+		else
+		{
+			if( (( y - m_pHitTestCache->m_Y[e]) * m_pHitTestCache->m_YMultiplier[e] ) -
+					(( x - m_pHitTestCache->m_X[e]) * m_pHitTestCache->m_XMultiplier[e] ) <= 0)
+			{
+				m_pHitTestCache->m_LastFailedEdge = e;
+				return TqFalse;
+			}
+		}
+
+		// move to next edge, wrapping to zero at four.
+		prev = e;
+		e = (e+1) & 3;
+	}
+
+	Depth = ( m_pHitTestCache->m_D - ( m_pHitTestCache->m_VecN.x() * vecP.x() ) -
+			( m_pHitTestCache->m_VecN.y() * vecP.y() ) ) * m_pHitTestCache->m_OneOverVecNZ;
+
+	return TqTrue;
+}
+
+//---------------------------------------------------------------------
+/** Cache some values needed for the point in poly test.
+ * This must be called prior to calling fContains() on a mpg.
+ */
+
+inline void CqMicroPolygon::CacheHitTestValues(CqHitTestCache* cache, CqVector3D* points)
+{
+	m_pHitTestCache = cache;
+
+	int j = 3;
+	for(int i=0; i<4; ++i)
+	{
+		cache->m_YMultiplier[i] = points[i].x() - points[j].x();
+		cache->m_XMultiplier[i] = points[i].y() - points[j].y();
+		cache->m_X[i] = points[j].x();
+		cache->m_Y[i] = points[j].y();
+		j = i;
+	}
+
+	// if the mpg is degenerate then we repeat edge c=>a so we still have four
+	// edges (it makes the test in fContains() simpler).
+	if(IsDegenerate())
+	{
+		for(int i=2; i<4; ++i)
+		{
+			cache->m_YMultiplier[i] = points[3].x() - points[1].x();
+			cache->m_XMultiplier[i] = points[3].y() - points[1].y();
+			cache->m_X[i] = points[1].x();
+			cache->m_Y[i] = points[1].y();
+		}
+	}
+
+	cache->m_VecN = (points[3] - points[0]) % (points[1] - points[0]);
+	cache->m_VecN.Unit();
+	cache->m_D = cache->m_VecN * points[3];
+	cache->m_OneOverVecNZ = 1.0 / cache->m_VecN.z();
+
+	cache->m_LastFailedEdge = 0;
+}
+
+void CqMicroPolygon::CacheHitTestValues(CqHitTestCache* cache)
+{
+	CqVector3D points[4] = { PointB(), PointC(), PointD(), PointA() };
+	CacheHitTestValues(cache, points);
+}
+
+
+// AGG - 19-6-04
+// this version moves the corners of the mpg for dof.
+// currently we don't use this as it's a fair bit slower than just offsetting
+// the sample position in the opposite direction (we need to call this for every
+// sample instead of just once). however, it is more correct than moving the
+// sample because the corners may move by different amounts. if I can work out
+// how to make it fast enough we should use this version.
+void CqMicroPolygon::CacheHitTestValuesDof(CqHitTestCache* cache, const CqVector2D& DofOffset, CqVector2D* coc)
+{
+	CqVector3D points[4];
+	points[0].x(PointB().x() - (coc[1].x() * DofOffset.x()));
+	points[0].y(PointB().y() - (coc[1].y() * DofOffset.y()));
+	points[0].z(PointB().z());
+	points[1].x(PointC().x() - (coc[2].x() * DofOffset.x()));
+	points[1].y(PointC().y() - (coc[2].y() * DofOffset.y()));
+	points[1].z(PointC().z());
+	points[2].x(PointD().x() - (coc[3].x() * DofOffset.x()));
+	points[2].y(PointD().y() - (coc[3].y() * DofOffset.y()));
+	points[2].z(PointD().z());
+	points[3].x(PointA().x() - (coc[0].x() * DofOffset.x()));
+	points[3].y(PointA().y() - (coc[0].y() * DofOffset.y()));
+	points[3].z(PointA().z());
+
+	CacheHitTestValues(cache, points);
 }
 
 
 CqVector2D CqMicroPolygon::ReverseBilinear( const CqVector2D& v )
 {
     CqVector2D kA, kB, kC, kD;
+    CqVector2D kResult;
+	TqBool flip = TqFalse;
 
     kA = CqVector2D( PointA() );
-    kB = CqVector2D( PointB() ) - kA;
-    kC = CqVector2D( PointD() ) - kA;
-    kD = CqVector2D( PointC() ) + kA - CqVector2D( PointB() ) - CqVector2D( PointD() );
+    kB = CqVector2D( PointB() );
+    kC = CqVector2D( PointD() );
+    kD = CqVector2D( PointC() );
+
+	if(fabs(kB.x() - kA.x()) < fabs(kC.x() - kA.x()) )
+	{
+		CqVector2D temp = kC;
+		kC = kB;
+		kB = temp;
+		//flip = TqTrue;
+	}
+
+    kD += kA - kB - kC;
+    kB -= kA;
+    kC -= kA;
 
     TqFloat fBCdet = kB.x() * kC.y() - kB.y() * kC.x();
     TqFloat fCDdet = kC.y() * kD.x() - kC.x() * kD.y();
@@ -1153,7 +1326,6 @@ CqVector2D CqMicroPolygon::ReverseBilinear( const CqVector2D& v )
     TqFloat fA = fCDdet;
     TqFloat fB = fADdet + fBCdet;
     TqFloat fC = fABdet;
-    CqVector2D kResult;
 
     if ( fabs( fA ) >= 1.0e-6 )
     {
@@ -1176,6 +1348,12 @@ CqVector2D CqMicroPolygon::ReverseBilinear( const CqVector2D& v )
         kResult.y( -fC / fB );
     }
     kResult.x( -( kDiff.x() + kResult.y() * kC.x() ) / ( kB.x() + kResult.y() * kD.x() ) );
+	if(flip)
+	{
+		TqFloat temp = kResult.x();
+		kResult.x(kResult.y());
+		kResult.y(temp);
+	}
 
     return ( kResult );
 }
@@ -1256,96 +1434,88 @@ TqBool CqMicroPolygon::Sample( const CqVector2D& vecSample, TqFloat& D, TqFloat
 /** Calculate the 2D boundary of this micropolygon,
  */
 
-CqBound CqMicroPolygon::GetTotalBound( TqBool fForce )
+void CqMicroPolygon::CalculateTotalBound()
 {
     CqVector3D * pP;
     m_pGrid->pVar(EnvVars_P) ->GetPointPtr( pP );
-    if ( fForce )
-    {
-        // Calculate the boundary, and store the indexes in the cache.
-        const CqVector3D& B = pP[ m_Index + 1 ];
-        const CqVector3D& C = pP[ m_Index + m_pGrid->uGridRes() + 2 ];
-        const CqVector3D& D = pP[ m_Index + m_pGrid->uGridRes() + 1 ];
 
-        TqShort BCMinX = 0;
-        TqShort BCMaxX = 0;
-        TqShort BCMinY = 0;
-        TqShort BCMaxY = 0;
-        TqShort BCMinZ = 0;
-        TqShort BCMaxZ = 0;
-        m_BoundCode = 0xe4;
-        TqInt TempIndexTable[ 4 ] = {	GetCodedIndex( m_BoundCode, 0 ),
-                                      GetCodedIndex( m_BoundCode, 1 ),
-                                      GetCodedIndex( m_BoundCode, 2 ),
-                                      GetCodedIndex( m_BoundCode, 3 ) };
-        if ( B.x() < pP[ TempIndexTable[ BCMinX ] ].x() ) BCMinX = 1;
-        if ( B.x() > pP[ TempIndexTable[ BCMaxX ] ].x() ) BCMaxX = 1;
-        if ( B.y() < pP[ TempIndexTable[ BCMinY ] ].y() ) BCMinY = 1;
-        if ( B.y() > pP[ TempIndexTable[ BCMaxY ] ].y() ) BCMaxY = 1;
-        if ( B.z() < pP[ TempIndexTable[ BCMinZ ] ].z() ) BCMinZ = 1;
-        if ( B.z() > pP[ TempIndexTable[ BCMaxZ ] ].z() ) BCMaxZ = 1;
+	// Calculate the boundary, and store the indexes in the cache.
+	const CqVector3D& B = pP[ m_Index + 1 ];
+	const CqVector3D& C = pP[ m_Index + m_pGrid->uGridRes() + 2 ];
+	const CqVector3D& D = pP[ m_Index + m_pGrid->uGridRes() + 1 ];
 
-        if ( C.x() < pP[ TempIndexTable[ BCMinX ] ].x() ) BCMinX = 2;
-        if ( C.x() > pP[ TempIndexTable[ BCMaxX ] ].x() ) BCMaxX = 2;
-        if ( C.y() < pP[ TempIndexTable[ BCMinY ] ].y() ) BCMinY = 2;
-        if ( C.y() > pP[ TempIndexTable[ BCMaxY ] ].y() ) BCMaxY = 2;
-        if ( C.z() < pP[ TempIndexTable[ BCMinZ ] ].z() ) BCMinZ = 2;
-        if ( C.z() > pP[ TempIndexTable[ BCMaxZ ] ].z() ) BCMaxZ = 2;
+	TqShort BCMinX = 0;
+	TqShort BCMaxX = 0;
+	TqShort BCMinY = 0;
+	TqShort BCMaxY = 0;
+	TqShort BCMinZ = 0;
+	TqShort BCMaxZ = 0;
+	m_BoundCode = 0xe4;
+	TqInt TempIndexTable[ 4 ] = {	GetCodedIndex( m_BoundCode, 0 ),
+								GetCodedIndex( m_BoundCode, 1 ),
+								GetCodedIndex( m_BoundCode, 2 ),
+								GetCodedIndex( m_BoundCode, 3 ) };
+	if ( B.x() < pP[ TempIndexTable[ BCMinX ] ].x() ) BCMinX = 1;
+	if ( B.x() > pP[ TempIndexTable[ BCMaxX ] ].x() ) BCMaxX = 1;
+	if ( B.y() < pP[ TempIndexTable[ BCMinY ] ].y() ) BCMinY = 1;
+	if ( B.y() > pP[ TempIndexTable[ BCMaxY ] ].y() ) BCMaxY = 1;
+	if ( B.z() < pP[ TempIndexTable[ BCMinZ ] ].z() ) BCMinZ = 1;
+	if ( B.z() > pP[ TempIndexTable[ BCMaxZ ] ].z() ) BCMaxZ = 1;
 
-        if ( !IsDegenerate() )
-        {
-            if ( D.x() < pP[ TempIndexTable[ BCMinX ] ].x() ) BCMinX = 3;
-            if ( D.x() > pP[ TempIndexTable[ BCMaxX ] ].x() ) BCMaxX = 3;
-            if ( D.y() < pP[ TempIndexTable[ BCMinY ] ].y() ) BCMinY = 3;
-            if ( D.y() > pP[ TempIndexTable[ BCMaxY ] ].y() ) BCMaxY = 3;
-            if ( D.z() < pP[ TempIndexTable[ BCMinZ ] ].z() ) BCMinZ = 3;
-            if ( D.z() > pP[ TempIndexTable[ BCMaxZ ] ].z() ) BCMaxZ = 3;
-        }
-        m_BoundCode = ( ( BCMinX & 0x3 ) |
-                        ( ( BCMinY & 0x3 ) << 2 ) |
-                        ( ( BCMinZ & 0x3 ) << 4 ) |
-                        ( ( BCMaxX & 0x3 ) << 6 ) |
-                        ( ( BCMaxY & 0x3 ) << 8 ) |
-                        ( ( BCMaxZ & 0x3 ) << 10 ) );
-    }
-    CqBound B( pP[ GetCodedIndex( m_BoundCode, 0 ) ].x(), pP[ GetCodedIndex( m_BoundCode, 1 ) ].y(), pP[ GetCodedIndex( m_BoundCode, 2 ) ].z(),
+	if ( C.x() < pP[ TempIndexTable[ BCMinX ] ].x() ) BCMinX = 2;
+	if ( C.x() > pP[ TempIndexTable[ BCMaxX ] ].x() ) BCMaxX = 2;
+	if ( C.y() < pP[ TempIndexTable[ BCMinY ] ].y() ) BCMinY = 2;
+	if ( C.y() > pP[ TempIndexTable[ BCMaxY ] ].y() ) BCMaxY = 2;
+	if ( C.z() < pP[ TempIndexTable[ BCMinZ ] ].z() ) BCMinZ = 2;
+	if ( C.z() > pP[ TempIndexTable[ BCMaxZ ] ].z() ) BCMaxZ = 2;
+
+	if ( !IsDegenerate() )
+	{
+		if ( D.x() < pP[ TempIndexTable[ BCMinX ] ].x() ) BCMinX = 3;
+		if ( D.x() > pP[ TempIndexTable[ BCMaxX ] ].x() ) BCMaxX = 3;
+		if ( D.y() < pP[ TempIndexTable[ BCMinY ] ].y() ) BCMinY = 3;
+		if ( D.y() > pP[ TempIndexTable[ BCMaxY ] ].y() ) BCMaxY = 3;
+		if ( D.z() < pP[ TempIndexTable[ BCMinZ ] ].z() ) BCMinZ = 3;
+		if ( D.z() > pP[ TempIndexTable[ BCMaxZ ] ].z() ) BCMaxZ = 3;
+	}
+	m_BoundCode = ( ( BCMinX & 0x3 ) |
+					( ( BCMinY & 0x3 ) << 2 ) |
+					( ( BCMinZ & 0x3 ) << 4 ) |
+					( ( BCMaxX & 0x3 ) << 6 ) |
+					( ( BCMaxY & 0x3 ) << 8 ) |
+					( ( BCMaxZ & 0x3 ) << 10 ) );
+
+    m_Bound = CqBound( pP[ GetCodedIndex( m_BoundCode, 0 ) ].x(), pP[ GetCodedIndex( m_BoundCode, 1 ) ].y(), pP[ GetCodedIndex( m_BoundCode, 2 ) ].z(),
                pP[ GetCodedIndex( m_BoundCode, 3 ) ].x(), pP[ GetCodedIndex( m_BoundCode, 4 ) ].y(), pP[ GetCodedIndex( m_BoundCode, 5 ) ].z() );
 
     // Adjust for DOF
     if ( QGetRenderContext() ->UsingDepthOfField() )
     {
-        const CqVector2D minZCoc = QGetRenderContext()->GetCircleOfConfusion( B.vecMin().z() );
-        const CqVector2D maxZCoc = QGetRenderContext()->GetCircleOfConfusion( B.vecMax().z() );
+        const CqVector2D minZCoc = QGetRenderContext()->GetCircleOfConfusion( m_Bound.vecMin().z() );
+        const CqVector2D maxZCoc = QGetRenderContext()->GetCircleOfConfusion( m_Bound.vecMax().z() );
         TqFloat cocX = MAX( minZCoc.x(), maxZCoc.x() );
         TqFloat cocY = MAX( minZCoc.y(), maxZCoc.y() );
 
-        B.vecMin().x( B.vecMin().x() - cocX );
-        B.vecMin().y( B.vecMin().y() - cocY );
-        B.vecMax().x( B.vecMax().x() + cocX );
-        B.vecMax().y( B.vecMax().y() + cocY );
+        m_Bound.vecMin().x( m_Bound.vecMin().x() - cocX );
+        m_Bound.vecMin().y( m_Bound.vecMin().y() - cocY );
+        m_Bound.vecMax().x( m_Bound.vecMax().x() + cocX );
+        m_Bound.vecMax().y( m_Bound.vecMax().y() + cocY );
     }
-
-    return ( B );
 }
 
 
 //---------------------------------------------------------------------
 /** Calculate the 2D boundary of this micropolygon,
- * \param fForce Boolean flag to force the recalculation of the cached bound.
  */
 
-CqBound CqMicroPolygonMotion::GetTotalBound( TqBool fForce )
+void CqMicroPolygonMotion::CalculateTotalBound()
 {
-    if ( fForce )
-    {
-        assert( NULL != m_Keys[ 0 ] );
+	assert( NULL != m_Keys[ 0 ] );
 
-        m_Bound = m_Keys[ 0 ] ->GetTotalBound();
-        std::vector<CqMovingMicroPolygonKey*>::iterator i;
-        for ( i = m_Keys.begin(); i != m_Keys.end(); i++ )
-            m_Bound.Encapsulate( ( *i ) ->GetTotalBound() );
-    }
-    return ( m_Bound );
+	m_Bound = m_Keys[ 0 ] ->GetTotalBound();
+	std::vector<CqMovingMicroPolygonKey*>::iterator i;
+	for ( i = m_Keys.begin(); i != m_Keys.end(); i++ )
+		m_Bound.Encapsulate( ( *i ) ->GetTotalBound() );
 }
 
 //---------------------------------------------------------------------
@@ -1353,45 +1523,70 @@ CqBound CqMicroPolygonMotion::GetTotalBound( TqBool fForce )
  */
 void CqMicroPolygonMotion::BuildBoundList()
 {
+    TqFloat opentime = QGetRenderContext() ->optCurrent().GetFloatOption( "System", "Shutter" ) [ 0 ];
+    TqFloat closetime = QGetRenderContext() ->optCurrent().GetFloatOption( "System", "Shutter" ) [ 1 ];
+    TqFloat shadingrate = pGrid() ->pAttributes() ->GetFloatAttribute( "System", "ShadingRate" ) [ 0 ];
+
 	m_BoundList.Clear();
 
     assert( NULL != m_Keys[ 0 ] );
 
-    CqBound start = m_Keys[ 0 ] ->GetTotalBound();
-    TqFloat	startTime = m_Times[ 0 ];
-    TqInt cTimes = m_Keys.size();
-    TqInt div = 0;
+	// compute an approximation of the distance travelled in raster space,
+	// we use this to guide how many sub-bounds to calcuate. note, it's much
+	// better for this to be fast than accurate, it's just a guide.
+	TqFloat dx = fabs(m_Keys.front()->m_Point0.x() - m_Keys.back()->m_Point0.x());
+	TqFloat dy = fabs(m_Keys.front()->m_Point0.y() - m_Keys.back()->m_Point0.y());
+	TqInt d = static_cast<int>((dx + dy) / shadingrate) + 1; // d is always >= 1
 
-	TqInt divisions = 10;
-    m_BoundList.SetSize( divisions * ( cTimes - 1 ));
-    for ( TqInt i = 1; i < cTimes; i++ )
-    {
-        CqBound end = m_Keys[ i ] ->GetTotalBound();
+	TqInt timeRanges = CqBucket::NumTimeRanges();
+	TqInt divisions = MIN(d, timeRanges);
+	TqFloat dt = (closetime - opentime) / divisions;
+	TqFloat time = opentime + dt;
+	TqInt startKey = 0;
+	TqInt endKey = 1;
+	CqBound bound = m_Keys[startKey]->GetTotalBound();
 
-        CqBound mid0( start );
-        CqBound mid1;
-        TqFloat endTime = m_Times[ i ];
-        TqFloat time = startTime;
+    m_BoundList.SetSize( divisions );
 
-        TqInt d;
-        // arbitary number of divisions, should be related to screen size of blur
-        TqFloat delta = 1.0f / static_cast<TqFloat>( divisions );
-		TqFloat pos = delta;
-        for ( d = 0; d < divisions; d++ )
-        {
-            mid1.vecMin() = pos * ( end.vecMin() - start.vecMin() ) + start.vecMin();
-            mid1.vecMax() = pos * ( end.vecMax() - start.vecMax() ) + start.vecMax();
-            
-			m_BoundList.Set( d + div, mid0.Combine( mid1 ), time );
-            time = pos * ( endTime - startTime ) + startTime;
-            mid0 = mid1;
-            pos += delta;
-        }
-        start = end;
-        startTime = endTime;
-        div += divisions;
-    }
-    m_BoundReady = TqTrue;
+	// create a bound for each time period.
+	for(TqInt i = 0; i < divisions; i++)
+	{
+		// find the fist key with a time greater than our end time.
+		while(time > m_Times[endKey] && endKey < m_Keys.size() - 1)
+			++endKey;
+
+		// interpolate between this key and the previous to get the
+		// bound at our end time.
+		TqInt endKey_1 = endKey - 1;
+		const CqBound& end0 = m_Keys[endKey_1]->GetTotalBound();
+		TqFloat end0Time = m_Times[endKey_1];
+		const CqBound& end1 = m_Keys[endKey]->GetTotalBound();
+		TqFloat end1Time = m_Times[endKey];
+
+		TqFloat mix = (time - end0Time) / (end1Time - end0Time);
+		CqBound mid(end0);
+		mid.vecMin() += mix * (end1.vecMin() - end0.vecMin());
+		mid.vecMax() += mix * (end1.vecMax() - end0.vecMax());
+
+		// combine with our starting bound.
+		bound.Encapsulate(mid);
+
+		// now combine the bound with any keys that fall between our start
+		// and end times.
+		while(startKey < endKey_1)
+		{
+			startKey++;
+			bound.Encapsulate(m_Keys[startKey]->GetTotalBound());
+		}
+
+		m_BoundList.Set( i, bound, time - dt );
+
+		// now set our new start to our current end ready for the next bound.
+		bound = mid;
+		time += dt;
+	}
+
+	m_BoundReady = TqTrue;
 }
 
 
@@ -1417,7 +1612,7 @@ TqBool CqMicroPolygonMotion::Sample( const CqVector2D& vecSample, TqFloat& D, Tq
             			CqString strTrimSense( "inside" );
             			if ( pattrTrimSense != 0 ) strTrimSense = pattrTrimSense[ 0 ];
             			TqBool bOutside = strTrimSense == "outside";
-             
+
             			CqVector2D vecUV = ReverseBilinear( vecSample );
              
             			TqFloat u, v;
@@ -1429,7 +1624,7 @@ TqBool CqMicroPolygonMotion::Sample( const CqVector2D& vecSample, TqFloat& D, Tq
             			pGrid() ->u() ->GetFloat( u, m_Index + 1 );
             			pGrid() ->v() ->GetFloat( v, m_Index + 1 );
             			CqVector2D uvB( u, v );
-             
+
             			pGrid() ->u() ->GetFloat( u, m_Index + pGrid() ->uGridRes() + 1 );
             			pGrid() ->v() ->GetFloat( v, m_Index + pGrid() ->uGridRes() + 1 );
             			CqVector2D uvC( u, v );
@@ -1602,6 +1797,8 @@ void CqMovingMicroPolygonKey::Initialise( const CqVector3D& vA, const CqVector3D
     m_N = ( vA - vvB ) % ( vvC - vvB );
     m_N.Unit();
     m_D = m_N * vA;
+
+	m_BoundReady = false;
 }
 
 
@@ -1686,32 +1883,35 @@ CqVector2D CqMovingMicroPolygonKey::ReverseBilinear( const CqVector2D& v )
 /** Calculate the 2D boundary of this micropolygon,
  */
 
-CqBound CqMovingMicroPolygonKey::GetTotalBound() const
+const CqBound& CqMovingMicroPolygonKey::GetTotalBound()
 {
-    CqBound Bound;
-    // Calculate the boundary, and store the indexes in the cache.
-    Bound.vecMin().x( MIN( m_Point0.x(), MIN( m_Point1.x(), MIN( m_Point2.x(), m_Point3.x() ) ) ) );
-    Bound.vecMin().y( MIN( m_Point0.y(), MIN( m_Point1.y(), MIN( m_Point2.y(), m_Point3.y() ) ) ) );
-    Bound.vecMin().z( MIN( m_Point0.z(), MIN( m_Point1.z(), MIN( m_Point2.z(), m_Point3.z() ) ) ) );
-    Bound.vecMax().x( MAX( m_Point0.x(), MAX( m_Point1.x(), MAX( m_Point2.x(), m_Point3.x() ) ) ) );
-    Bound.vecMax().y( MAX( m_Point0.y(), MAX( m_Point1.y(), MAX( m_Point2.y(), m_Point3.y() ) ) ) );
-    Bound.vecMax().z( MAX( m_Point0.z(), MAX( m_Point1.z(), MAX( m_Point2.z(), m_Point3.z() ) ) ) );
+	if(m_BoundReady)
+		return m_Bound;
+
+	// Calculate the boundary, and store the indexes in the cache.
+    m_Bound.vecMin().x( MIN( m_Point0.x(), MIN( m_Point1.x(), MIN( m_Point2.x(), m_Point3.x() ) ) ) );
+    m_Bound.vecMin().y( MIN( m_Point0.y(), MIN( m_Point1.y(), MIN( m_Point2.y(), m_Point3.y() ) ) ) );
+    m_Bound.vecMin().z( MIN( m_Point0.z(), MIN( m_Point1.z(), MIN( m_Point2.z(), m_Point3.z() ) ) ) );
+    m_Bound.vecMax().x( MAX( m_Point0.x(), MAX( m_Point1.x(), MAX( m_Point2.x(), m_Point3.x() ) ) ) );
+    m_Bound.vecMax().y( MAX( m_Point0.y(), MAX( m_Point1.y(), MAX( m_Point2.y(), m_Point3.y() ) ) ) );
+    m_Bound.vecMax().z( MAX( m_Point0.z(), MAX( m_Point1.z(), MAX( m_Point2.z(), m_Point3.z() ) ) ) );
 
 	// Adjust for DOF
 	if ( QGetRenderContext() ->UsingDepthOfField() )
 	{
-		const CqVector2D minZCoc = QGetRenderContext()->GetCircleOfConfusion( Bound.vecMin().z() );
-		const CqVector2D maxZCoc = QGetRenderContext()->GetCircleOfConfusion( Bound.vecMax().z() );
+		const CqVector2D minZCoc = QGetRenderContext()->GetCircleOfConfusion( m_Bound.vecMin().z() );
+		const CqVector2D maxZCoc = QGetRenderContext()->GetCircleOfConfusion( m_Bound.vecMax().z() );
 		TqFloat cocX = MAX( minZCoc.x(), maxZCoc.x() );
 		TqFloat cocY = MAX( minZCoc.y(), maxZCoc.y() );
 
-		Bound.vecMin().x( Bound.vecMin().x() - cocX );
-		Bound.vecMin().y( Bound.vecMin().y() - cocY );
-		Bound.vecMax().x( Bound.vecMax().x() + cocX );
-		Bound.vecMax().y( Bound.vecMax().y() + cocY );
+		m_Bound.vecMin().x( m_Bound.vecMin().x() - cocX );
+		m_Bound.vecMin().y( m_Bound.vecMin().y() - cocY );
+		m_Bound.vecMax().x( m_Bound.vecMax().x() + cocX );
+		m_Bound.vecMax().y( m_Bound.vecMax().y() + cocY );
 	}
 
-    return ( Bound );
+	m_BoundReady = true;
+    return ( m_Bound );
 }
 
 
