@@ -365,6 +365,28 @@ public:
 private:
 };
 
+//----------------------------------------------------------------------
+/** \struct CqHitTestCache
+ * struct holding data used during the point in poly test.
+ */
+
+struct CqHitTestCache
+{
+	// these 3 are used in calculating the interpolated depth value
+	CqVector3D m_VecN;
+	TqFloat m_OneOverVecNZ;
+	TqFloat m_D;
+
+	// these 4 hold values used in doing the edge tests. 1 of each for each edge.
+	TqFloat m_YMultiplier[4];
+	TqFloat m_XMultiplier[4];
+	TqFloat m_X[4];
+	TqFloat m_Y[4];
+
+	// this holds the index of the last edge that failed an edge test, chances
+	// are it will fail on the next sample as well, so we test this edge first.
+	TqInt	m_LastFailedEdge;
+};
 
 //----------------------------------------------------------------------
 /** \class CqMicroPolygon
@@ -416,7 +438,7 @@ public:
     /** Get the pointer to the grid this micropoly came from.
      * \return Pointer to the CqMicroPolyGrid.
      */
-    virtual CqMicroPolyGridBase* pGrid() const
+    CqMicroPolyGridBase* pGrid() const
     {
         return ( m_pGrid );
     }
@@ -463,17 +485,23 @@ public:
         return ( colRes );
     }
 
+    /** Calculate the bound of the micropoly.
+	 */
+	virtual void CalculateTotalBound();
 
     // Overridables
     /** Get the bound of the micropoly.
-     * \param fForce Flag indicating do not get the stored bound, but recalculate it.
      * \return CqBound representing the conservative bound.
      */
-    virtual	CqBound	GetTotalBound( TqBool fForce = TqFalse );
+    virtual	CqBound GetTotalBound()
+	{
+		return m_Bound;
+	}
+
     /** Get the bound of the micropoly.
      * \return CqBound representing the conservative bound.
      */
-    virtual const CqBound	GetTotalBound() const
+    virtual const CqBound GetTotalBound() const
     {
         return ( GetTotalBound() );
     }
@@ -481,7 +509,7 @@ public:
     {
         return ( 1 );
     }
-    virtual	CqBound	SubBound( TqInt iIndex, TqFloat& time )
+    virtual	CqBound SubBound( TqInt iIndex, TqFloat& time )
     {
         time = 0.0f;
         return ( GetTotalBound() );
@@ -528,8 +556,15 @@ public:
         return ( ( m_Flags & MicroPolyFlags_Trimmed ) != 0 );
     }
 
+	virtual TqBool IsMoving()
+	{
+		return TqFalse;
+	}
 
     virtual TqBool	fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const;
+	void	CacheHitTestValues(CqHitTestCache* cache, CqVector3D* points);
+	void	CacheHitTestValues(CqHitTestCache* cache);
+	void	CacheHitTestValuesDof(CqHitTestCache* cache, const CqVector2D& DofOffset, CqVector2D* coc);
     void	Initialise();
     CqVector2D ReverseBilinear( const CqVector2D& v );
 
@@ -578,11 +613,15 @@ protected:
         }
     }
     TqLong	m_IndexCode;
+    CqBound	m_Bound;					///< Stored bound.
+
     TqLong	m_BoundCode;
     CqMicroPolyGridBase*	m_pGrid;		///< Pointer to the donor grid.
     TqInt	m_Index;		///< Index within the donor grid.
 
     TqShort	m_Flags;		///< Bitvector of general flags, using EqMicroPolyFlags as bitmasks.
+
+	CqHitTestCache* m_pHitTestCache; // struct to hold cached values used in the point-in-poly test
 private:
     CqMicroPolygon( const CqMicroPolygon& From )
 {}
@@ -610,7 +649,7 @@ public:
 public:
     TqBool	fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const;
 
-    CqBound	GetTotalBound() const;
+    const CqBound&	GetTotalBound();
     void	Initialise( const CqVector3D& vA, const CqVector3D& vB, const CqVector3D& vC, const CqVector3D& vD );
     CqVector2D ReverseBilinear( const CqVector2D& v );
 
@@ -620,6 +659,9 @@ public:
     CqVector3D	m_Point3;
     CqVector3D	m_N;			///< The normal to the micropoly.
     TqFloat	m_D;				///< Distance of the plane from the origin, used for calculating sample depth.
+
+	CqBound m_Bound;
+	TqBool	m_BoundReady;
 }
 ;
 
@@ -648,7 +690,8 @@ public:
 
     // Overrides from CqMicroPolygon
     virtual TqBool	fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const;
-    virtual	CqBound	GetTotalBound( TqBool fForce = TqFalse );
+	virtual void CalculateTotalBound();
+	virtual	CqBound	GetTotalBound() { return m_Bound; }
     virtual const CqBound	GetTotalBound() const
     {
         return ( m_Bound );
@@ -675,8 +718,13 @@ public:
     {
         m_fTrimmed = TqTrue;
     }
+
+	virtual TqBool IsMoving()
+	{
+		return TqTrue;
+	}
+
 private:
-    CqBound	m_Bound;					///< Stored bound.
     CqBoundList	m_BoundList;			///< List of bounds to get a tighter fit.
     TqBool	m_BoundReady;				///< Flag indicating the boundary has been initialised.
     std::vector<TqFloat> m_Times;
