@@ -49,7 +49,7 @@ START_NAMESPACE( Aqsis )
 extern IqDDManager* CreateDisplayDriverManager();
 extern IqRaytrace* CreateRaytracer();
 
-static CqShaderRegister * pOShaderRegister = NULL;
+//static CqShaderRegister * pOShaderRegister = NULL;
 
 CqRenderer* pCurrRenderer = 0;
 
@@ -75,8 +75,8 @@ CqRenderer::CqRenderer() :
         m_pImageBuffer( 0 ),
         m_Mode( RenderMode_Image ),
         m_fSaveGPrims( TqFalse ),
-        m_OutputDataOffset(7),		// Cs, Os, z
-        m_OutputDataTotalSize(7),	// Cs, Os, z
+        m_OutputDataOffset(9),		// Cs, Os, z, coverage, a
+        m_OutputDataTotalSize(9),	// Cs, Os, z, coverage, a
         m_FrameNo( 0 ),
 		m_bObjectOpen(TqFalse),
         m_pErrorHandler( &RiErrorPrint ),
@@ -92,11 +92,9 @@ CqRenderer::CqRenderer() :
 
     // Initialize the default attributes, transform and camera transform
     m_pAttrDefault  = new CqAttributes;
-    m_pTransDefault = new CqTransform;
-    m_pTransCamera  = new CqTransform;
     ADDREF( m_pAttrDefault );
-    ADDREF( m_pTransDefault );
-    ADDREF( m_pTransCamera );
+    m_pTransDefault = CqTransformPtr( new CqTransform );
+    m_pTransCamera  = CqTransformPtr( new CqTransform );
 
     // Initialise the array of coordinate systems.
     m_aCoordSystems.resize( CoordSystem_Last );
@@ -168,16 +166,6 @@ CqRenderer::~CqRenderer()
         RELEASEREF( m_pAttrDefault );
         m_pAttrDefault = NULL;
     }
-    if ( m_pTransDefault )
-    {
-        RELEASEREF( m_pTransDefault );
-        m_pTransDefault = NULL;
-    }
-    if ( m_pTransCamera )
-    {
-        RELEASEREF( m_pTransCamera );
-        m_pTransCamera = NULL;
-    }
 
 	if( m_pRaytracer )	// MGC: MEMLEAK_FIX
 	{
@@ -521,6 +509,13 @@ TqFloat	CqRenderer::Time() const
         return ( QGetRenderContext() ->optCurrent().GetFloatOptionWrite( "System", "Shutter" ) [ 0 ] );
 }
 
+
+TqInt CqRenderer::bucketCount()
+{
+	return(pImage()->cXBuckets() * pImage()->cYBuckets() );
+}
+
+
 //----------------------------------------------------------------------
 /** Advance the current shutter time, only valid within motion blocks.
  */
@@ -578,7 +573,7 @@ CqAttributes* CqRenderer::pattrWriteCurrent()
 /** Return a pointer to the current transform.
  */
 
-const CqTransform* CqRenderer::ptransCurrent()
+CqTransformPtr CqRenderer::ptransCurrent()
 {
     if ( m_pconCurrent )
         return ( m_pconCurrent->ptransCurrent() );
@@ -588,16 +583,51 @@ const CqTransform* CqRenderer::ptransCurrent()
 
 
 //----------------------------------------------------------------------
-/** Return a writable pointer to the current transform.
+/** Modify the current transformation.
  */
 
-CqTransform* CqRenderer::ptransWriteCurrent()
+#if 0
+CqTransformPtr CqRenderer::ptransWriteCurrent()
 {
     if ( m_pconCurrent )
         return ( m_pconCurrent->ptransWriteCurrent() );
     else
         return ( m_pTransDefault );
 }
+#endif
+
+void	CqRenderer::ptransSetTime( const CqMatrix& matTrans )
+{
+	if ( !m_pconCurrent )
+	{
+		throw 0;
+	}
+
+	CqTransformPtr newTrans( new CqTransform( m_pconCurrent->ptransCurrent(), Time(), matTrans, CqTransform::Set() ) );
+	m_pconCurrent->ptransSetCurrent( newTrans );
+}
+
+void	CqRenderer::ptransSetCurrentTime( const CqMatrix& matTrans )
+{
+	if ( !m_pconCurrent )
+	{
+		throw 0;
+	}
+
+	CqTransformPtr newTrans( new CqTransform( m_pconCurrent->ptransCurrent(), Time(), matTrans, CqTransform::SetCurrent() ) );
+	m_pconCurrent->ptransSetCurrent( newTrans );
+}
+
+void	CqRenderer::ptransConcatCurrentTime( const CqMatrix& matTrans )
+{
+	if ( !m_pconCurrent )
+	{
+		throw 0;
+	}
+
+	CqTransformPtr newTrans( new CqTransform( m_pconCurrent->ptransCurrent(), Time(), matTrans, CqTransform::ConcatCurrent() ) );
+	m_pconCurrent->ptransSetCurrent( newTrans );
+}
 
 
 //----------------------------------------------------------------------
@@ -614,7 +644,7 @@ void CqRenderer::RenderWorld()
 
     pImage() ->RenderImage();
 
-    m_pDDManager->CloseDisplays();
+	m_pDDManager->CloseDisplays();
 }
 
 
@@ -647,8 +677,8 @@ void CqRenderer::Initialise()
 
     // Clear the output data entries
     m_OutputDataEntries.clear();
-    m_OutputDataOffset = 7;		// Cs, Os, z
-    m_OutputDataTotalSize = 7;	// Cs, Os, z
+    m_OutputDataOffset = 9;		// Cs, Os, depth, coverage, a
+    m_OutputDataTotalSize = 9;	// Cs, Os, depth, coverage, a
 }
 
 
@@ -1006,18 +1036,19 @@ void CqRenderer::AddParameterDecl( const char* strName, const char* strType )
 //---------------------------------------------------------------------
 /** Register a shader of the specified type with the specified name.
  */
-
+#if 0
 void CqRenderer::RegisterShader( const char* strName, EqShaderType type, IqShader* pShader )
 {
     assert( pShader );
     m_Shaders.LinkLast( new CqShaderRegister( strName, type, pShader ) );
 }
+#endif
 
 
 //---------------------------------------------------------------------
 /** Find a shader of the specified type with the specified name.
  */
-
+#if 0
 CqShaderRegister* CqRenderer::FindShader( const char* strName, EqShaderType type )
 {
     // Search the register list.
@@ -1037,13 +1068,121 @@ CqShaderRegister* CqRenderer::FindShader( const char* strName, EqShaderType type
     }
     return ( 0 );
 }
+#endif
 
+//---------------------------------------------------------------------
+/** Returns a pointer to the default surface.
+ */
+boost::shared_ptr<IqShader> CqRenderer::getDefaultSurfaceShader()
+{
+        // construct a key to index the default surface
+        CqShaderKey key( "_def_", Type_Surface );
+
+        // check for the shader in the existing map
+        boost::shared_ptr<IqShader> pMapCheck = 
+                CreateShader( "_def_", Type_Surface );
+        if (pMapCheck)
+                return pMapCheck;
+
+        // insert the default surface template into the map
+        boost::shared_ptr<IqShader> pRet( new CqShaderVM() );
+        CqShaderVM* pShader = static_cast<CqShaderVM*>( pRet.get() );
+        pShader->SetstrName( "_def_" );
+        pShader->DefaultSurface();
+        pShader->matCurrent() = matCurrent(Time());
+        pShader->PrepareDefArgs();
+        m_Shaders[key] = pRet;
+        
+        // return a clone of the default surface template
+        return boost::shared_ptr<IqShader>( pRet->Clone() );
+        
+}
+
+//---------------------------------------------------------------------
+/** Find a shader of the specified type with the specified name.
+ * If not found, try to load one.
+ */
+boost::shared_ptr<IqShader> CqRenderer::CreateShader( 
+        const char* strName, EqShaderType type )
+{
+        // construct the key which is used to index the shader
+        CqShaderKey key( strName, type );
+        
+        // first, look for the shader of the appropriate type and name in the
+        //  map of shader "templates"
+        if ( m_Shaders.find(key) != m_Shaders.end() )
+        {
+                // the shader template is present, so return its clone
+                return boost::shared_ptr<IqShader>( m_Shaders[key]->Clone() );
+        }
+
+        // we now create the shader...
+
+        // search in the current directory first
+        CqString strFilename( strName );
+        strFilename += RI_SHADER_EXTENSION;
+        CqRiFile SLXFile( strFilename.c_str(), "shader" );
+        if ( SLXFile.IsValid() )
+        {
+                boost::shared_ptr<IqShader> pRet( new CqShaderVM() );
+        
+                CqShaderVM* pShader = static_cast<CqShaderVM*>( pRet.get() );
+                const CqString* poptDSOPath = QGetRenderContext()->
+                    optCurrent().GetStringOption( "searchpath", "dsolibs" );
+                pShader->SetDSOPath( poptDSOPath );
+
+                CqString strRealName( SLXFile.strRealName() );
+                std::cerr << info << "Loading shader \"" << strName 
+                          << "\" from file \"" << strRealName.c_str()
+                          << "\"" << std::endl;
+
+                pShader->SetstrName( strName );
+                pShader->LoadProgram( SLXFile );
+                
+                // add the shader to the map as a template and return its 
+                //  clone
+                m_Shaders[key] = pRet;
+                return boost::shared_ptr<IqShader>( pRet->Clone() );
+        }
+        else
+        {
+                if ( 
+                        (strcmp( strName, "null" )  != 0) &&
+                        (strcmp( strName, "_def_" ) != 0)
+                )
+                {
+                        CqString strError;
+                        strError.Format( "Shader \"%s\" not found",
+                                strName ? strName : "" );
+                        std::cerr << error << strError.c_str() << std::endl;
+                }
+                if ( type == Type_Surface )
+                {
+                        boost::shared_ptr<IqShader> pRet( new CqShaderVM() );
+                
+                        CqShaderVM* pShader = static_cast<CqShaderVM*>(
+                                pRet.get() );
+                        pShader->SetstrName( "null" );
+                        pShader->DefaultSurface();
+
+                        // add the shader to the map and return its clone
+                        m_Shaders[key] = pRet;
+                        return boost::shared_ptr<IqShader>( pRet->Clone() );
+                }
+                else
+                {
+                        // the boost::shared_ptr analogue of return NULL:
+                        return boost::shared_ptr<IqShader>();
+                }
+        }
+        
+}
 
 //---------------------------------------------------------------------
 /** Find a shader of the specified type with the specified name.
  * If not found, try and load one.
  */
-
+#if 0
 IqShader* CqRenderer::CreateShader( const char* strName, EqShaderType type )
 {
     CqShaderRegister * pReg = NULL;
@@ -1070,12 +1209,8 @@ IqShader* CqRenderer::CreateShader( const char* strName, EqShaderType type )
             const CqString *poptDSOPath = QGetRenderContext()->optCurrent().GetStringOption( "searchpath","dsolibs" );
             pShader->SetDSOPath( poptDSOPath );
 
-            const TqInt* poptVerbose = QGetRenderContext() ->optCurrent().GetIntegerOption( "statistics", "verbose" );
             CqString strRealName( SLXFile.strRealName() );
-            if ( poptVerbose )
-            {
-                std::cerr << info << "Loading shader \"" << strName << "\" from file \"" << strRealName.c_str() << "\"" << std::endl;
-            }
+			std::cerr << info << "Loading shader \"" << strName << "\" from file \"" << strRealName.c_str() << "\"" << std::endl;
 
             pShader->SetstrName( strName );
             pShader->LoadProgram( SLXFile );
@@ -1103,6 +1238,7 @@ IqShader* CqRenderer::CreateShader( const char* strName, EqShaderType type )
         }
     }
 }
+#endif
 
 
 
@@ -1282,6 +1418,7 @@ TqInt CqRenderer::RegisterOutputData( const char* name )
 
         DataEntry.m_Offset = m_OutputDataOffset;
         DataEntry.m_NumSamples = NumSamples;
+		DataEntry.m_Type = Decl.m_Type;
         m_OutputDataOffset += NumSamples;
         m_OutputDataTotalSize += NumSamples;
 
@@ -1337,6 +1474,28 @@ TqInt CqRenderer::OutputDataSamples( const char* name )
 }
 
 
+TqInt CqRenderer::OutputDataType( const char* name )
+{
+    SqParameterDeclaration Decl;
+    try
+    {
+        Decl = FindParameterDecl( name );
+    }
+    catch( XqException e )
+    {
+        std::cerr << error << e.strReason().c_str() << std::endl;
+        return(-1);
+    }
+    if( Decl.m_Type != type_invalid )
+    {
+        std::map<std::string, SqOutputDataEntry>::iterator entry = m_OutputDataEntries.find( Decl.m_strName );
+        if( entry != m_OutputDataEntries.end() )
+            return( entry->second.m_Type );
+    }
+    return( 0 );
+}
+
+
 CqObjectInstance* CqRenderer::OpenNewObjectInstance()
 {
 	assert( !m_bObjectOpen );
