@@ -616,16 +616,33 @@ void CqSubdivision2::SubdivideFace(CqLath* pFace, std::vector<CqLath*>& apSubFac
     // First make sure that the appropriate neighbour facets have been subdivided if this is >0 level face.
     if( pFace->pParentFacet() )
     {
-        std::vector<CqLath*> aQff;
-        std::vector<CqLath*> apSubFaces2;
-        pFace->pParentFacet()->Qff( aQff );
-        std::vector<CqLath*>::iterator iF;
-        for( iF = aQff.begin(); iF != aQff.end(); iF++ )
+		// loop through all our neighbour faces.
+		// we don't use Qff here because we can handle multiple copies
+		// of each face faster than it can.
+		std::vector<CqLath*> parentVertices;
+		pFace->pParentFacet()->Qfv( parentVertices );
+
+		std::vector<CqLath*>::iterator vertexIt;
+		std::vector<CqLath*>::iterator vertexEnd = parentVertices.end();
+		for( vertexIt = parentVertices.begin(); vertexIt != vertexEnd; ++vertexIt )
 		{
-		    if( NULL == (*iF)->pFaceVertex() )
-			    SubdivideFace(*iF, apSubFaces2);
+			CqLath* vertex = *vertexIt;
+			std::vector<CqLath*> vertexFaces;
+			vertex->Qvf( vertexFaces );
+
+			std::vector<CqLath*>::iterator faceIt;
+			std::vector<CqLath*>::iterator faceEnd = vertexFaces.end();
+			for( faceIt = vertexFaces.begin(); faceIt != faceEnd; ++faceIt )
+			{
+				CqLath* face = (*faceIt);
+				if( NULL == face->pFaceVertex() )
+				{
+					std::vector<CqLath*> dummySubFaces;
+					SubdivideFace(face, dummySubFaces);
+				}
+			}
 		}
-    }
+	}
 
     std::vector<CqLath*> aQfv;
     std::vector<TqInt> aVertices;
@@ -886,7 +903,9 @@ CqBound	CqSurfaceSubdivisionPatch::Bound() const
     assert( pTopology()->pPoints() );
     assert( pFace() );
 
-    // First make sure that the appropriate neighbour facets have been subdivided if this is >0 level face.
+    CqBound B;
+
+	// First make sure that the appropriate neighbour facets have been subdivided if this is >0 level face.
     if( pFace()->pParentFacet() )
     {
         std::vector<CqLath*> aQff;
@@ -894,10 +913,13 @@ CqBound	CqSurfaceSubdivisionPatch::Bound() const
         pFace()->pParentFacet()->Qff( aQff );
         std::vector<CqLath*>::iterator iF;
         for( iF = aQff.begin(); iF != aQff.end(); iF++ )
-            pTopology()->SubdivideFace(*iF, apSubFaces2);
+		{
+			CqLath* face = *iF;
+			if (NULL == face->pFaceVertex())
+				pTopology()->SubdivideFace(face, apSubFaces2);
+		}
     }
 
-    CqBound B;
 
     // Get the laths of the surrounding faces.
     std::vector<CqLath*> aQff;
@@ -960,12 +982,12 @@ CqMicroPolyGridBase* CqSurfaceSubdivisionPatch::Dice()
 CqMicroPolyGridBase* CqSurfaceSubdivisionPatch::DiceExtract()
 {
 	// Dice rate table			  0  1  2  3  4  5  6  7  8  9  10 11 12 13 14 15 16
-    static TqInt aDiceSizes[] = { 0, 0, 1, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 4, 4 };
+    static TqInt aDiceSizes[] = { 0, 0, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 4, 4, 4 };
 	assert( pTopology() );
     assert( pTopology()->pPoints() );
     assert( pFace() );
 
-    TqInt dicesize = MAX(m_uDiceSize, m_vDiceSize);
+    TqInt dicesize = MIN(MAX(m_uDiceSize, m_vDiceSize), 16);
 
 	TqInt sdcount = aDiceSizes[ dicesize ];
     dicesize = 1 << sdcount;
@@ -989,7 +1011,7 @@ CqMicroPolyGridBase* CqSurfaceSubdivisionPatch::DiceExtract()
             apSubFace2.clear();
             std::vector<CqLath*>::iterator iSF;
             for( iSF = apSubFace1.begin(); iSF != apSubFace1.end(); iSF++ )
-            {
+			{
                 // Subdivide this face, storing the resulting new face indices.
                 std::vector<CqLath*> apSubFaceTemp;
                 pTopology()->SubdivideFace( (*iSF), apSubFaceTemp );
@@ -1139,7 +1161,7 @@ CqMicroPolyGridBase* CqSurfaceSubdivisionPatch::DiceExtract()
 
 
 
-static void StoreDiceAPVar( IqShader* pShader, CqParameter* pParam, TqUint ivA, TqUint indexA )
+static void StoreDiceAPVar( const boost::shared_ptr<IqShader>& pShader, CqParameter* pParam, TqUint ivA, TqUint indexA )
 {
     // Find the argument
     IqShaderData * pArg = pShader->FindArgument( pParam->strName() );
@@ -1210,7 +1232,7 @@ static void StoreDiceAPVar( IqShader* pShader, CqParameter* pParam, TqUint ivA,
 
 void CqSurfaceSubdivisionPatch::StoreDice( CqMicroPolyGrid* pGrid, const boost::shared_ptr<CqPolygonPoints>& pPoints, TqInt iParam, TqInt iFVParam, TqInt iData)
 {
-    TqInt lUses = Uses();
+    TqInt lUses = m_Uses;
     TqInt lDone = 0;
 
     if ( USES( lUses, EnvVars_P ) )
@@ -1264,14 +1286,14 @@ void CqSurfaceSubdivisionPatch::StoreDice( CqMicroPolyGrid* pGrid, const boost::
     for ( iUP = pPoints->aUserParams().begin(); iUP != pPoints->aUserParams().end(); iUP++ )
     {
         /// \todo: Must transform point/vector/normal/matrix parameter variables from 'object' space to current before setting.
-        IqShader* pShader;
-		if ( (pShader=pGrid->pAttributes() ->pshadSurface(QGetRenderContextI()->Time())) != NULL )
+        boost::shared_ptr<IqShader> pShader;
+		if ( pShader=pGrid->pAttributes() ->pshadSurface(m_Time) )
             StoreDiceAPVar( pShader, ( *iUP ), iParam, iData );
 
-        if ( (pShader=pGrid->pAttributes() ->pshadDisplacement(QGetRenderContextI()->Time())) != NULL )
+        if ( pShader=pGrid->pAttributes() ->pshadDisplacement(m_Time) )
             StoreDiceAPVar( pShader, ( *iUP ), iParam, iData );
 
-        if ( (pShader=pGrid->pAttributes() ->pshadAtmosphere(QGetRenderContextI()->Time())) != NULL )
+        if ( pShader=pGrid->pAttributes() ->pshadAtmosphere(m_Time) )
             StoreDiceAPVar( pShader, ( *iUP ), iParam, iData );
     }
 }
@@ -1303,7 +1325,7 @@ TqInt CqSurfaceSubdivisionPatch::Split( std::vector<boost::shared_ptr<CqBasicSur
         aiVertices.push_back( pPoint->VertexIndex() );
         aiFVertices.push_back( pPoint->FaceVertexIndex() );
         // 0,3
-        pPoint = pPoint->cv()->ccf();
+       pPoint = pPoint->cv()->ccf();
         aiVertices.push_back( pPoint->VertexIndex() );
         aiFVertices.push_back( pPoint->FaceVertexIndex() );
         // 1,0
@@ -1570,27 +1592,28 @@ TqBool CqSurfaceSubdivisionPatch::Diceable()
 
     m_SplitDir = ( uLen > vLen ) ? SplitDir_U : SplitDir_V;
 
-    // TODO: Should ensure powers of half to prevent cracking.
     uLen = MAX( ROUND( uLen ), 1 );
     vLen = MAX( ROUND( vLen ), 1 );
 
     m_uDiceSize = static_cast<TqInt>( uLen );
     m_vDiceSize = static_cast<TqInt>( vLen );
 
-    // Ensure power of 2 to avoid cracking
-    m_uDiceSize = CEIL_POW2( m_uDiceSize );
-    m_vDiceSize = CEIL_POW2( m_vDiceSize );
+	// Note Subd surfaces always have a power of 2 dice rate because they
+	// are diced by recursive subdivision. Hence no need to set it explicitly.
 
-    if ( uLen < FLT_EPSILON || vLen < FLT_EPSILON )
+	if ( uLen < FLT_EPSILON || vLen < FLT_EPSILON )
     {
         m_fDiscard = TqTrue;
         return ( TqFalse );
     }
 
-    TqFloat gs = 16.0f;
+	// because splitting to a bicubic patch is so much faster than dicing by
+	// recursive subdivision, the grid size is made smaller than usual to give
+	// us more chance to break regular parts off as a patch.
+    TqFloat gs = 8.0f;
     const TqFloat* poptGridSize = QGetRenderContext() ->optCurrent().GetFloatOption( "System", "SqrtGridSize" );
     if( poptGridSize )
-        gs = poptGridSize[0];
+        gs = poptGridSize[0] / 2.0f;
 
     if ( m_uDiceSize > gs) return TqFalse;
     if ( m_vDiceSize > gs) return TqFalse;
@@ -1619,10 +1642,16 @@ TqBool CqSubdivision2::CanUsePatch( CqLath* pFace )
         if( (*iFV)->cQvv() != 4 )
             return( TqFalse );
 
-        // Check if all vertices smooth.
-        if( EdgeSharpness((*iFV)) != 0.0f ||
-                CornerSharpness((*iFV)) != 0.0f )
-            return( TqFalse );
+        // Check if all edges incident on the face vertices are smooth.
+        std::vector<CqLath*> aQve;
+        (*iFV)->Qve(aQve);
+        std::vector<CqLath*>::iterator iVE;
+        for( iVE = aQve.begin(); iVE!=aQve.end(); iVE++ )
+        {
+            if( EdgeSharpness((*iVE)) != 0.0f ||
+                    CornerSharpness((*iVE)) != 0.0f )
+                return( TqFalse );
+        }
 
         // Check if no internal boundaries.
         CqLath* pEnd = (*iFV)->cv();
@@ -1644,17 +1673,6 @@ TqBool CqSubdivision2::CanUsePatch( CqLath* pFace )
     {
         if( (*iFF)->cQfv() != 4 )
             return( TqFalse );
-
-        std::vector<CqLath*> aQfv;
-        (*iFF)->Qfv(aQfv);
-        std::vector<CqLath*>::iterator iFV;
-        for( iFV = aQfv.begin(); iFV!=aQfv.end(); iFV++ )
-        {
-            // Check if all vertices smooth.
-            if( EdgeSharpness((*iFV)) != 0.0f ||
-                    CornerSharpness((*iFV)) != 0.0f )
-                return( TqFalse );
-        }
     }
 
     // Finally check if the "facevarying" indexes match, as patches can't have
