@@ -132,63 +132,28 @@ TqPuchar CqTextureMapBuffer::AllocSegment( TqUlong width, TqUlong height, TqInt
  */
 TqInt CqTextureMap::Convert( CqString &strName )
 {
+
+    const CqString extension = strName.substr(strName.rfind(".")).substr(1);
+#ifdef	AQSIS_SYSTEM_POSIX
+    const CqString plugin_path = DEFAULT_PLUGIN_PATH "/" + extension + "2tif.so";
+#elif	AQSIS_SYSTEM_WIN32
+	char acPath[256];
+	if ( GetModuleFileName( NULL, acPath, 256 ) != 0) 
+	{
+		// guaranteed file name of at least one character after path
+		*( strrchr( acPath, '\\' ) + 1 ) = '\0';
+	}
+    CqString plugin_path = acPath;
+	plugin_path.append( CqString("/" + extension + "2tif.dll") );
+#endif
+    const CqString plugin_function = extension + "2tif";
+
     TqInt result = 0;
-    TqInt lenght = 0;
-    TqChar library[ 1024 ];
-    TqChar function[ 1024 ];
+    CqConverter* const plug = new CqConverter( "", const_cast<char*>(plugin_path.c_str()), const_cast<char*>(plugin_function.c_str()) );
     char * ( *convert ) ( const char * );
-    char *ext = NULL;
-    char *tiff = NULL;
-
-    convert = NULL;
-
-    char *aqsis_home = getenv( "AQSIS_BASE_PATH" );
-
-    if ( aqsis_home == NULL )
-#if defined(AQSIS_SYSTEM_POSIX) && !defined(AQSIS_SYSTEM_MACOSX)
-        aqsis_home = BASE_PATH;
-#else
-        aqsis_home = ".";
-#endif
-
-    ext = ( TqPchar ) strName.c_str();
-    if ( ext && *ext )
-        lenght = strlen( ext );
-
-    if ( lenght > 4 )
-    {
-
-        /* find the extension in the string
-         * than try to match which extension goes with which dll 
-         **/
-        ext = strstr( &ext[ lenght - 5 ], "." );
-        if ( ext == NULL ) return 0;
-        ext++;
-    }
-    else
-    {
-        return 0;
-    }
-
-    sprintf( function, "%s2tif", ext );
-#ifdef AQSIS_SYSTEM_WIN32
-    /*********************************/
-    /* Get the dynamic library on NT */
-    /*********************************/
-    sprintf( library, "%s\\procedures\\%s.dll", aqsis_home, function );
-#else
-    /***********************************/
-    /* Get the shared library on UNIX  */
-    /***********************************/
-    sprintf( library, "%s/lib/lib%s.so", aqsis_home, function );
-#endif
-    CqConverter *plug = new CqConverter( "", library, function );
     if ( ( convert = ( char * ( * ) (const  char * s ) ) plug->Function() ) != NULL )
     {
-
-#ifdef DEBUG
-        std::cout << "strName: " << strName.c_str() << std::endl;
-#endif
+	char* tiff = 0;
         if ( ( tiff = convert (  strName.c_str() ) ) != NULL )
         {
             strName = tiff;
@@ -490,6 +455,7 @@ CqTextureMapBuffer* CqTextureMap::GetBuffer( TqUlong s, TqUlong t, TqInt directo
 
 void CqTextureMap::GetSample( TqFloat u1, TqFloat v1, TqFloat u2, TqFloat v2, std::valarray<TqFloat>& val )
 {
+    QGetRenderContext() ->Stats().TextureMapTimer().Start(); 
     // u/v is average of sample positions.
     TqFloat u = ( ( u2 - u1 ) * 0.5f ) + u1;
     TqFloat v = ( ( v2 - v1 ) * 0.5f ) + v1;
@@ -649,6 +615,7 @@ void CqTextureMap::GetSample( TqFloat u1, TqFloat v1, TqFloat u2, TqFloat v2, st
 
     }
 
+    QGetRenderContext() ->Stats().TextureMapTimer().Stop(); 
 }
 
 //----------------------------------------------------------------------
@@ -807,13 +774,13 @@ void CqTextureMap::ImageFilterVal( CqTextureMapBuffer* pData, TqInt x, TqInt y,
     if ( directory )
     {
         TqInt i, j;
+        TqFloat yfilt = -(directory - 0.5f);
         for ( j = 0; j < ydelta; j++ )
         {
+	        TqFloat xfilt = -(directory - 0.5f);
             for ( i = 0; i < xdelta; i++ )
             {
                 /* find the filter value */
-                TqFloat xfilt = i - directory;
-                TqFloat yfilt = j - directory;
                 mul = ( *pFilter ) ( xfilt, yfilt, xdelta, ydelta );
 
                 /* find the value in the original image */
@@ -826,7 +793,10 @@ void CqTextureMap::ImageFilterVal( CqTextureMapBuffer* pData, TqInt x, TqInt y,
 
                 /* accumulate the ponderation factor */
                 div += mul;
+
+				xfilt += 1.0f;
             }
+			yfilt += 1.0f;
         }
 
         /* use the accumulated ponderation factor */
@@ -872,6 +842,7 @@ void CqTextureMap::Open()
 
     // Now open it as a tiff file.
     m_pImage = TIFFOpen( strRealName.c_str(), "r" );
+    std::cerr << info << "TextureMap: \"" << strRealName.c_str() << "\" is open" << std::endl;
     if ( m_pImage )
     {
         TqPchar pFormat = 0;
@@ -887,9 +858,13 @@ void CqTextureMap::Open()
         TIFFGetField( m_pImage, TIFFTAG_SAMPLESPERPIXEL, &samplesperpixel );
         m_SamplesPerPixel = samplesperpixel;
         uint16 sampleformat;
-        TIFFGetField( m_pImage, TIFFTAG_SAMPLEFORMAT, &sampleformat );
+        TIFFGetFieldDefaulted( m_pImage, TIFFTAG_SAMPLEFORMAT, &sampleformat );
         m_SampleFormat = sampleformat;
 
+        uint16 bitspersample;
+        TIFFGetFieldDefaulted( m_pImage, TIFFTAG_BITSPERSAMPLE, &bitspersample );
+        m_BitsPerSample = bitspersample;
+
         TIFFGetField( m_pImage, TIFFTAG_PIXAR_TEXTUREFORMAT, &pFormat );
         TIFFGetField( m_pImage, TIFFTAG_PIXAR_WRAPMODES, &pModes );
 
@@ -921,7 +896,7 @@ void CqTextureMap::Open()
 
         /* Second test; is it containing enough directories for us */
         TqInt min = MIN(m_XRes, m_YRes );
-        TqInt directory = (TqInt)  log((double) min)/log((double) 2.0);
+        TqInt directory = static_cast<TqInt>(log((double) min)/log((double) 2.0));
         bMipMap &= TIFFSetDirectory(m_pImage, directory - 1);
 
 
@@ -929,7 +904,7 @@ void CqTextureMap::Open()
 
 
         /* Support for 3delight, AIR, BMRT, RDC, PIXIE MipMap files.
-         * Aqsis is not bound to have exact multiples of 2 on height, lenght.
+         * Aqsis is not bound to have exact multiples of 2 on height, length.
          * The Format of 3Delight, AIR, BMRT and RDC is more "Plain Texture"/MipMap.
          * What is preventing us to load their files was the format description file as 
          * MipMap differ from our format description not the way they store their information.
@@ -1136,11 +1111,7 @@ void CqTextureMap::WriteImage( TIFF* ptex, TqPuchar raster, TqUlong width, TqUlo
     TqChar version[ 80 ];
     TIFFCreateDirectory( ptex );
 
-#if defined(AQSIS_SYSTEM_WIN32) || defined(AQSIS_SYSTEM_MACOSX)
     sprintf( version, "%s %s", STRNAME, VERSION_STR );
-#else
-    sprintf( version, "%s %s", STRNAME, VERSION );
-#endif
     TIFFSetField( ptex, TIFFTAG_SOFTWARE, ( uint32 ) version );
     TIFFSetField( ptex, TIFFTAG_IMAGEWIDTH, width );
     TIFFSetField( ptex, TIFFTAG_IMAGELENGTH, length );
@@ -1171,11 +1142,7 @@ void CqTextureMap::WriteImage( TIFF* ptex, TqFloat *raster, TqUlong width, TqUlo
     TqChar version[ 80 ];
     TIFFCreateDirectory( ptex );
 
-#if defined(AQSIS_SYSTEM_WIN32) || defined(AQSIS_SYSTEM_MACOSX)
     sprintf( version, "%s %s", STRNAME, VERSION_STR );
-#else
-    sprintf( version, "%s %s", STRNAME, VERSION );
-#endif
     TIFFSetField( ptex, TIFFTAG_SOFTWARE, ( uint32 ) version );
     TIFFSetField( ptex, TIFFTAG_IMAGEWIDTH, width );
     TIFFSetField( ptex, TIFFTAG_IMAGELENGTH, length );
@@ -1198,6 +1165,39 @@ void CqTextureMap::WriteImage( TIFF* ptex, TqFloat *raster, TqUlong width, TqUlo
 }
 
 
+//----------------------------------------------------------------------
+/** Write an image to an open TIFF file in the current directory as straight storage.
+ * as 16 bit int values
+ */
+
+void CqTextureMap::WriteImage( TIFF* ptex, TqUshort *raster, TqUlong width, TqUlong length, TqInt samples, TqInt compression, TqInt quality )
+{
+    TqChar version[ 80 ];
+    TIFFCreateDirectory( ptex );
+
+    sprintf( version, "%s %s", STRNAME, VERSION_STR );
+    TIFFSetField( ptex, TIFFTAG_SOFTWARE, ( uint32 ) version );
+    TIFFSetField( ptex, TIFFTAG_IMAGEWIDTH, width );
+    TIFFSetField( ptex, TIFFTAG_IMAGELENGTH, length );
+    TIFFSetField( ptex, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG );
+    TIFFSetField( ptex, TIFFTAG_BITSPERSAMPLE, 16 );
+    TIFFSetField( ptex, TIFFTAG_SAMPLESPERPIXEL, samples );
+    TIFFSetField( ptex, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT );
+    TIFFSetField( ptex, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_UINT );
+    TIFFSetField( ptex, TIFFTAG_COMPRESSION, compression ); /* COMPRESSION_DEFLATE */
+    TIFFSetField( ptex, TIFFTAG_ROWSPERSTRIP, 1 );
+    TIFFSetField( ptex, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB );
+
+    TqUshort *pdata = raster;
+    for ( TqUlong i = 0; i < length; i++ )
+    {
+        TIFFWriteScanline( ptex, pdata, i );
+        pdata += ( width * samples );
+    }
+    TIFFWriteDirectory( ptex );
+}
+
+
 //----------------------------------------------------------------------
 /** Write an image to an open TIFF file in the current directory as tiled storage.
  * determine the size and type from the buffer.
@@ -1218,6 +1218,12 @@ void CqTextureMap::WriteImage( TIFF* ptex, CqTextureMapBuffer* pBuffer, TqInt co
             WriteImage( ptex, static_cast<TqFloat*>( pBuffer->pVoidBufferData() ), pBuffer->Width(), pBuffer->Height(), pBuffer->Samples(), compression, quality );
             break;
         }
+
+    case BufferType_Int16:
+        {
+            WriteImage( ptex, static_cast<TqUshort*>( pBuffer->pVoidBufferData() ), pBuffer->Width(), pBuffer->Height(), pBuffer->Samples(), compression, quality );
+            break;
+        }
     }
 }
 
@@ -1241,6 +1247,12 @@ void CqTextureMap::WriteTileImage( TIFF* ptex, CqTextureMapBuffer* pBuffer, TqUl
             WriteTileImage( ptex, static_cast<TqFloat*>( pBuffer->pVoidBufferData() ), pBuffer->Width(), pBuffer->Height(), twidth, theight, pBuffer->Samples(), compression, quality );
             break;
         }
+
+    case BufferType_Int16:
+        {
+            WriteTileImage( ptex, static_cast<TqUshort*>( pBuffer->pVoidBufferData() ), pBuffer->Width(), pBuffer->Height(), twidth, theight, pBuffer->Samples(), compression, quality );
+            break;
+        }
     }
 }
 
@@ -1254,11 +1266,7 @@ void CqTextureMap::WriteTileImage( TIFF* ptex, TqFloat *raster, TqUlong width, T
 {
     //TIFFCreateDirectory(ptex);
     TqChar version[ 80 ];
-#if defined(AQSIS_SYSTEM_WIN32) || defined(AQSIS_SYSTEM_MACOSX)
     sprintf( version, "%s %s", STRNAME, VERSION_STR );
-#else
-    sprintf( version, "%s %s", STRNAME, VERSION );
-#endif
     TIFFSetField( ptex, TIFFTAG_SOFTWARE, ( uint32 ) version );
     TIFFSetField( ptex, TIFFTAG_IMAGEWIDTH, width );
     TIFFSetField( ptex, TIFFTAG_IMAGELENGTH, length );
@@ -1307,6 +1315,66 @@ void CqTextureMap::WriteTileImage( TIFF* ptex, TqFloat *raster, TqUlong width, T
     }
 }
 
+
+//----------------------------------------------------------------------
+/** Write an image to an open TIFF file in the current directory as tiled storage.
+ * as 16 bit int values
+ */
+
+void CqTextureMap::WriteTileImage( TIFF* ptex, TqUshort *raster, TqUlong width, TqUlong length, TqUlong twidth, TqUlong tlength, TqInt samples, TqInt compression, TqInt quality )
+{
+    //TIFFCreateDirectory(ptex);
+    TqChar version[ 80 ];
+    sprintf( version, "%s %s", STRNAME, VERSION_STR );
+    TIFFSetField( ptex, TIFFTAG_SOFTWARE, ( uint32 ) version );
+    TIFFSetField( ptex, TIFFTAG_IMAGEWIDTH, width );
+    TIFFSetField( ptex, TIFFTAG_IMAGELENGTH, length );
+    TIFFSetField( ptex, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG );
+    TIFFSetField( ptex, TIFFTAG_BITSPERSAMPLE, 16 );
+    TIFFSetField( ptex, TIFFTAG_SAMPLESPERPIXEL, samples );
+    TIFFSetField( ptex, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT );
+    TIFFSetField( ptex, TIFFTAG_TILEWIDTH, twidth );
+    TIFFSetField( ptex, TIFFTAG_TILELENGTH, tlength );
+    TIFFSetField( ptex, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_UINT );
+    TIFFSetField( ptex, TIFFTAG_COMPRESSION, compression );
+
+
+    TqInt tsize = twidth * tlength;
+    TqInt tperrow = ( width + twidth - 1 ) / twidth;
+    TqUshort* ptile = static_cast<TqUshort*>( _TIFFmalloc( tsize * samples * sizeof( TqUshort ) ) );
+
+    if ( ptile != NULL )
+    {
+        TqInt ctiles = tperrow * ( ( length + tlength - 1 ) / tlength );
+        TqInt itile;
+        for ( itile = 0; itile < ctiles; itile++ )
+        {
+            TqInt x = ( itile % tperrow ) * twidth;
+            TqInt y = ( itile / tperrow ) * tlength;
+            TqUshort* ptdata = raster + ( ( y * width ) + x ) * samples;
+            // Clear the tile to black.
+            memset( ptile, 0, tsize * samples * sizeof( TqUshort ) );
+            for ( TqUlong i = 0; i < tlength; i++ )
+            {
+                for ( TqUlong j = 0; j < twidth; j++ )
+                {
+                    if ( ( x + j ) < width && ( y + i ) < length )
+                    {
+                        TqInt ii;
+                        for ( ii = 0; ii < samples; ii++ )
+                            ptile[ ( i * twidth * samples ) + ( ( ( j * samples ) + ii ) ) ] = ptdata[ ( ( j * samples ) + ii ) ];
+                    }
+                }
+                ptdata += ( width * samples );
+            }
+            TIFFWriteTile( ptex, ptile, x, y, 0, 0 );
+        }
+        TIFFWriteDirectory( ptex );
+
+    }
+}
+
+
 //----------------------------------------------------------------------
 /** Write an image to an open TIFF file in the current directory as tiled storage.
  * as unsigned char values
@@ -1315,11 +1383,7 @@ void CqTextureMap::WriteTileImage( TIFF* ptex, TqFloat *raster, TqUlong width, T
 void CqTextureMap::WriteTileImage( TIFF* ptex, TqPuchar raster, TqUlong width, TqUlong length, TqUlong twidth, TqUlong tlength, TqInt samples, TqInt compression, TqInt quality )
 {
     TqChar version[ 80 ];
-#if defined(AQSIS_SYSTEM_WIN32) || defined(AQSIS_SYSTEM_MACOSX)
     sprintf( version, "%s %s", STRNAME, VERSION_STR );
-#else
-    sprintf( version, "%s %s", STRNAME, VERSION );
-#endif
     TIFFSetField( ptex, TIFFTAG_SOFTWARE, ( uint32 ) version );
     TIFFSetField( ptex, TIFFTAG_IMAGEWIDTH, width );
     TIFFSetField( ptex, TIFFTAG_IMAGELENGTH, length );
